﻿using Correo.Abstractions;
using Correo.Abstractions.Extensions;
using Correo.NetSmtpClient.Extensions;
using Correo.NetSmtpClient.Models;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using System;
using System.Net;
using System.Threading;
using System.Threading.Tasks;

namespace Correo.NetSmtpClient
{
    internal class NetSmtpClient : IMailer, IDisposable
    {
        private readonly IOptions<SmtpClientOptions> _optionsAccessor;
        private readonly System.Net.Mail.SmtpClient _smtpClient;
        private readonly ILogger<NetSmtpClient> _logger;

        public NetSmtpClient(IOptions<SmtpClientOptions> optionsAccessor, ILogger<NetSmtpClient> logger)
        {
            _optionsAccessor = optionsAccessor;
            _logger = logger;

            var options = _optionsAccessor.Value;

            _smtpClient = new System.Net.Mail.SmtpClient { Host = options.Server, Port = options.Port, EnableSsl = options.EnableSsl };
            if (options.UseAuthentication)
            {
                if (string.IsNullOrEmpty(options.Authentication.Domain))
                    _smtpClient.Credentials = new NetworkCredential(options.Authentication.UserName, options.Authentication.Password);
                else
                    _smtpClient.Credentials = new NetworkCredential(options.Authentication.UserName, options.Authentication.Password, options.Authentication.Domain);
            }
            else
                _smtpClient.UseDefaultCredentials = true;

            if (options.TrustServer)
                ServicePointManager.ServerCertificateValidationCallback = (sender, certificate, chain, sslPolicyErrors) => true;
        }

        public void Dispose()
        {
            _smtpClient?.Dispose();
            GC.SuppressFinalize(this);
        }

        public void SendEmail(EmailMessage message)
        {
            var mailMessage = message.ToMailMessage();
            _smtpClient.Send(mailMessage);
            Log(message);
        }

        public async Task SendEmailAsync(EmailMessage message, CancellationToken token = default)
        {
            var mailMessage = message.ToMailMessage();
            await _smtpClient.SendMailAsync(mailMessage, token);
            Log(message);
        }

        private void Log(EmailMessage message)
            => _logger.LogInformation($"Email sent: Subject: {message.Subject}; From: {message.From.Address}; To: {message.To.Log()}; Cc: {message.Cc.Log()}; Bcc: {message.Bcc.Log()};");
    }
}
